/*
 * Copyright 2025 IBM Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

export const ServerConnection = {
  makeSettings: jest.fn(() => ({
    baseUrl: 'http://localhost:8888/'
  })),
  makeRequest: jest.fn(),
  NetworkError: class extends Error {},
  ResponseError: class extends Error {
    constructor(
      public response: Response,
      message: string
    ) {
      super(message);
    }
  }
};
